package summarizer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class FMeasureGraphCalculator implements GraphSummaryCalculator {

	private Graph originalGraph;
	private OntologyData ontologyData;
	private double rAlpha;
	private double rBeta;
	private double fAlpha;
	private int relevantNodesLimit;
	private int summarySizeReference;
	private int summarySizeDelta;

	public FMeasureGraphCalculator() {
		this.rAlpha = 0.5;
		this.rBeta = 0.5;
		this.fAlpha = 0.5;
	}

	public FMeasureGraphCalculator(Graph originalGraph, int relevantNodesLimit,
			int summarySizeReference, int symmarySizeDelta) {
		this();
		this.originalGraph = originalGraph;
		this.relevantNodesLimit = relevantNodesLimit;
		this.summarySizeReference = summarySizeReference;
		this.summarySizeDelta = symmarySizeDelta;
	}

	public FMeasureGraphCalculator(Graph originalGraph) {
		this();
		this.originalGraph = originalGraph;
		this.relevantNodesLimit = (int) Math
				.round(originalGraph.getSize(true) * 0.20);
		this.summarySizeReference = this.relevantNodesLimit;
		this.summarySizeDelta = (int) Math
				.round(this.summarySizeReference * 0.50);
	}

	public int getSummarySizeDelta() {
		return summarySizeDelta;
	}

	public void setSummarySizeDelta(int summarySizeDelta) {
		this.summarySizeDelta = summarySizeDelta;
	}

	public OntologyData getOntologyData() {
		return ontologyData;
	}

	public void setOntologyData(OntologyData ontologyData) {
		this.ontologyData = ontologyData;
	}

	public Graph getOriginalGraph() {
		return originalGraph;
	}

	public void setOriginalGraph(Graph originalGraph) {
		this.originalGraph = originalGraph;
	}

	public double getRAlpha() {
		return rAlpha;
	}

	public void setRAlpha(double alpha) {
		rAlpha = alpha;
	}

	public double getRBeta() {
		return rBeta;
	}

	public void setRBeta(double beta) {
		rBeta = beta;
	}

	public double getFAlpha() {
		return fAlpha;
	}

	public void setFAlpha(double alpha) {
		fAlpha = alpha;
	}

	public int getRelevantNodesLimit() {
		return relevantNodesLimit;
	}

	public void setRelevantNodesLimit(int relevantNodesLimit) {
		this.relevantNodesLimit = relevantNodesLimit;
	}

	public int getSummarySizeReference() {
		return summarySizeReference;
	}

	public void setSummarySizeReference(int summarySizeReference) {
		this.summarySizeReference = summarySizeReference;
	}

	public int getSymmarySizeDelta() {
		return summarySizeDelta;
	}

	public void setSymmarySizeDelta(int symmarySizeDelta) {
		this.summarySizeDelta = symmarySizeDelta;
	}

	/**
	 * 
	 * @see summarizer.GraphSummaryCalculator#getJointNodesGraph(summarizer.Graph,
	 *      summarizer.Edge)
	 */
	public Graph getJointNodesGraph(Graph originalGraph, Edge edge) {
		Graph newGraph = new Graph();
		newGraph.setName(originalGraph.getName());
		Set<Node> oldNodes = originalGraph.getNodes();
		Set<Edge> oldEdges = originalGraph.getEdges();

		Node node = new Node(originalGraph.getAvaiableNodeID());
		node.setLabel("[" + edge.getSource() + ", " + edge.getTarget() + "]");
		if (edge.getSource().isRelevant() && edge.getTarget().isRelevant()) {
			node.setRelevant(true);
			double newRelevance = (edge.getSource().getRelevance() + edge
					.getTarget().getRelevance()) / 2.0;
			node.setRelevance(newRelevance);
		}
		Graph subgraph = new Graph();
		subgraph.setName("Subgraph");
		subgraph.addNode(edge.getSource());
		subgraph.addNode(edge.getTarget());
		if (!edge.isAutoEdge()) {
			subgraph.addEdge(edge);
		}
		Set<Edge> autoedgesSource = originalGraph
				.getAutoEdges(edge.getSource());
		Set<Edge> autoedgesTarget = originalGraph
				.getAutoEdges(edge.getTarget());
		if (autoedgesSource.size() > 0) {
			subgraph.addEdges(autoedgesSource);
		}
		if (autoedgesTarget.size() > 0) {
			subgraph.addEdges(autoedgesTarget);
		}

		node.setSubgraph(subgraph);
		node.setSourceConnectionPoints(this.getSourceConnectedPoints(
				originalGraph, edge));
		node.setTargetConnectionPoints(this.getTargetConnectedPoints(
				originalGraph, edge));
		newGraph.addNode(node);
		for (Node n : oldNodes) {
			if (!n.equals(edge.getSource()) && !n.equals(edge.getTarget())) {
				newGraph.addNode(n);
			}
		}
		for (Edge e : oldEdges) {
			if (!e.equals(edge) && !e.isInverseOf(edge)) {
				if (e.isAutoEdge()) {
					boolean hasSourceOrTarget = (e.getSource().equals(
							edge.getSource()) || e.getSource().equals(
							edge.getTarget()));
					if (!hasSourceOrTarget) {
						newGraph.addEdge(e);
					}
				} else {
					boolean sourceContainsAnyNode = e.getSource().equals(
							edge.getSource())
							|| (e.getSource().equals(edge.getTarget()));
					boolean targetContainAnyNode = e.getTarget().equals(
							edge.getSource())
							|| (e.getTarget().equals(edge.getTarget()));
					if (sourceContainsAnyNode && !targetContainAnyNode) {
						Edge edg = new Edge(node, e.getTarget());
						edg.addLabel(e.getLabel());
						newGraph.addEdge(edg);
					}
					if (!sourceContainsAnyNode && targetContainAnyNode) {
						Edge edg = new Edge(e.getSource(), node);
						edg.addLabel(e.getLabel());
						newGraph.addEdge(edg);
					}
					if (!sourceContainsAnyNode && !targetContainAnyNode) {
						newGraph.addEdge(e);
					}
				}
			}
		}
		return newGraph;
	}

	public Graph unjoinGraph(Graph jGraph) {
		Graph uGraph = jGraph.getCopy();
		while (uGraph.containsSubgraph()) {
			Set<Node> nodes = uGraph.getNodes();
			for (Node n : nodes) {
				if (n.isSubgraph()) {
					Set<Node> uNodes = n.getSubgraph().getNodes();
					Set<Edge> uEdges = n.getSubgraph().getEdges();
					uGraph.getEdges().addAll(uEdges);
					uGraph.getNodes().addAll(uNodes);

					Node sourceNode = null;
					Node targetNode = null;
					for (Edge sEdge : uEdges) {
						if (!sEdge.isAutoEdge()) {
							sourceNode = sEdge.getSource();
							targetNode = sEdge.getTarget();
							break;
						}
					}

					Set<Edge> edgesFromN = this.getGraphEdgesByNode(uGraph, n);
					Set<Node> scNodes = n.getSourceConnectionPoints();
					Set<Node> tcNodes = n.getTargetConnectionPoints();
					for (Edge e : edgesFromN) {
						Node candidate = e.getOtherPoint(n);
						if (scNodes.contains(candidate)) {
							e.setOtherPointSource(n, targetNode);
						} else if (tcNodes.contains(candidate)) {
							e.setOtherPointSource(n, sourceNode);
						}
					}
					uGraph.getNodes().remove(n);
					break;
				}
			}
		}
		return uGraph;
	}

	private Set<Node> getTargetConnectedPoints(Graph originalGraph, Edge edge) {
		Set<Node> cp = new HashSet<Node>();
		Set<Edge> edges = this.getGraphEdgesByNode(originalGraph, edge
				.getSource());
		edges.remove(edge);
		for (Edge e : edges) {
			cp.add(e.getOtherPoint(edge.getSource()));
		}
		return cp;
	}

	private Set<Node> getSourceConnectedPoints(Graph originalGraph, Edge edge) {
		Set<Node> cp = new HashSet<Node>();
		Set<Edge> edges = this.getGraphEdgesByNode(originalGraph, edge
				.getTarget());
		edges.remove(edge);
		for (Edge e : edges) {
			cp.add(e.getOtherPoint(edge.getTarget()));
		}
		return cp;
	}

	public int getNodeCentrality(Graph graph, Node node) {
		int centrality = 0;
		Set<Edge> edges = graph.getEdges();
		Set<Edge> edgesComputed = new HashSet<Edge>();
		for (Edge edge : edges) {
			if (edge.getSource().equals(node) || edge.getTarget().equals(node)) {
				if (!edge.isAutoEdge() && !edgesComputed.contains(edge)) {
					edgesComputed.add(edge);
					centrality++;
				}
			}
		}
		return centrality;
	}

	public double getNodeNormalizedCentrality(Graph graph, Node node) {
		double normalizedCentrality = 0.0;
		double numerator = this.getNodeCentrality(graph, node);
		double denominator = graph.getNodes().size() - 1;
		normalizedCentrality = numerator / denominator;
		return normalizedCentrality;
	}

	private double getEdgeCentrality(Graph graph, Edge edge) {
		double sourceCentrality = this.getNodeCentrality(graph, edge
				.getSource());
		double targetCentrality = this.getNodeCentrality(graph, edge
				.getTarget());
		return (sourceCentrality + targetCentrality - 2) / 2.0;
	}

	public double getEdgeNormalizedCentrality(Graph graph, Edge edge) {
		double normalizedCentrality = 0.0;
		double numerator = this.getEdgeCentrality(graph, edge);
		double denominator = graph.getNodes().size() - 2;
		normalizedCentrality = numerator / denominator;
		return normalizedCentrality;
	}

	public int getNodeFrequency(Graph graph, Node node,
			OntologyData ontologyData) {
		int frequency = 0;
		if (ontologyData != null) {
			if (graph.contains(node)) {
				frequency = ontologyData.getFrequency(node.getLabel());
			}
		}
		return frequency;
	}

	public double getNodeNormalizedFrequency(Graph graph, Node node,
			OntologyData ontologyData) {
		double normalizedFrequency = 0.0;
		double numerator = this.getNodeFrequency(graph, node, ontologyData);
		if (ontologyData != null) {
			double denominator = ontologyData.getHigherFrequency();
			normalizedFrequency = numerator / denominator;
		}
		return normalizedFrequency;
	}

	public double getNodeRelevance(Graph graph, Node node,
			OntologyData ontologyData, double rAlpha, double rBeta) {
		double relevance = rAlpha
				* this.getNodeNormalizedCentrality(graph, node) + rBeta
				* this.getNodeNormalizedFrequency(graph, node, ontologyData);
		return relevance;
	}

	public double getAverageNodeRelevance(Graph graph,
			OntologyData ontologyData, double alpha, double beta) {
		double average = 0.0;
		Set<Node> nodes = graph.getNodes();
		for (Node node : nodes) {
			average += this.getNodeRelevance(graph, node, ontologyData, alpha,
					beta);
		}
		average = average / nodes.size();
		return average;
	}

	public void calculateNodesRelevancies(Graph graph,
			OntologyData ontologyData, double alpha, double beta, int limit) {
		double average = this.getAverageNodeRelevance(graph, ontologyData,
				alpha, beta);
		Set<Node> nodes = graph.getNodes();
		for (Node node : nodes) {
			double nodeRelevance = this.getNodeRelevance(graph, node,
					ontologyData, alpha, beta);
			node.setRelevance(nodeRelevance);
			if (nodeRelevance >= average) {
				node.setRelevant(true);
			}
		}
		if (limit > 0) {
			List<Node> rNodes = graph.getRelevantNodes();
			if (rNodes.size() > limit) {
				int i = 0;
				for (Node n : rNodes) {
					if (i >= limit) {
						n.setRelevant(false);
					}
					i++;
				}
			}
		}
	}

	public Set<Graph> getAllPaths(Graph original, Node source, Node target) {
		original.hideAutoEdges();
		Set<Graph> subgraphs = new HashSet<Graph>();
		Set<Edge> edgesFromA = this.getGraphEdgesByNode(original, source);
		for (Edge edge : edgesFromA) {
			Graph pathGraph = new Graph();
			pathGraph.addNode(source);
			pathGraph.addNode(edge.getOtherPoint(source));
			pathGraph.addEdge(edge);
			Set<Graph> paths = new HashSet<Graph>();
			this.getPathGraph(original, pathGraph, paths, edge
					.getOtherPoint(source), target, source);
			if (pathGraph != null) {
				subgraphs.addAll(paths);
			}
		}
		this.updateAutoEdges(subgraphs, original.getAutoEdges());
		original.showAutoEdges();
		return subgraphs;
	}

	public void updateAutoEdges(Set<Graph> graphs, Set<Edge> autoEdges) {
		for (Graph g : graphs) {
			for (Edge e : autoEdges) {
				if (e.isAutoEdge() && g.getNodes().contains(e.getSource())) {
					g.addEdge(e);
				}
			}
		}
	}

	private Graph getPathGraph(Graph original, Graph g, Set<Graph> paths,
			Node localSource, Node globalTarget, Node globalSource) {
		if (g == null) {
			return null;
		}
		if (g.isPathFinded()) {
			return g;
		}
		if (localSource.equals(globalTarget)) {
			g.setPathFinded(true);
			return g;
		} else {
			Set<Edge> edgesFromLocalSource = this.getGraphEdgesByNode(original,
					localSource);
			Set<Edge> edgesFromLocalGraph = this.getGraphEdgesByNode(g,
					localSource);
			Set<Edge> edgesFromGlobalSource = this.getGraphEdgesByNode(
					original, globalSource);
			edgesFromLocalSource.removeAll(edgesFromLocalGraph);
			edgesFromLocalSource.removeAll(edgesFromGlobalSource);
			if ((edgesFromLocalSource.size() == 0)
					|| (g.getSize(false) > summarySizeReference
							+ summarySizeDelta)) {
				return null;
			}
			boolean ok = false;
			for (Edge edge : edgesFromLocalSource) {
				g.addNode(localSource);
				g.addNode(edge.getOtherPoint(localSource));
				g.addEdge(edge);
				if (this
						.getPathGraph(original, g, paths, edge
								.getOtherPoint(localSource), globalTarget,
								globalSource) == null) {
					if (this.getGraphEdgesByNode(g, localSource).size() == 1) {
						g.getNodes().remove(localSource);
					}
					if (this.getGraphEdgesByNode(g,
							edge.getOtherPoint(localSource)).size() == 1) {
						g.getNodes().remove(edge.getOtherPoint(localSource));
					}
					g.getEdges().remove(edge);
				} else {
					ok = true;
					if ((g != null) && g.isPathFinded()) {
						paths.add(g.getCopy());
						g.setPathFinded(false);
						if (this.getGraphEdgesByNode(g, localSource).size() == 1) {
							g.getNodes().remove(localSource);
						}
						if (this.getGraphEdgesByNode(g,
								edge.getOtherPoint(localSource)).size() == 1) {
							g.getNodes()
									.remove(edge.getOtherPoint(localSource));
						}
						g.getEdges().remove(edge);
						ok = false;
					} else {
						g = this.getPathGraph(original, g, paths, edge
								.getOtherPoint(localSource), globalTarget,
								globalSource);
					}
				}
			}
			if (!ok) {
				return null;
			}
			return g;
		}
	}

	public Set<Edge> getGraphEdgesByNode(Graph g, Node node) {
		Set<Edge> edges = new HashSet<Edge>();
		for (Edge edge : g.getEdges()) {
			if (edge.getSource().equals(node) || edge.getTarget().equals(node)) {
				edges.add(edge);
			}
		}
		return edges;
	}

	public double getPrecision(Graph g) {
		double precision = 0.0;
		Set<Node> nodes = g.getNodes();
		double numberRelevants = 0.0;
		for (Node n : nodes) {
			if (!n.isSubgraph()) {
				if (n.isRelevant()) {
					numberRelevants += 1.0;
				}
			} else {
				Set<Node> subnodes = n.getSubgraph().getNodes();
				for (Node nn : subnodes) {
					if (nn.isRelevant()) {
						numberRelevants += 1.0;
					}
				}
			}
		}
		double total = 0.0;
		for (Node n : nodes) {
			if (n.isSubgraph()) {
				total += n.getSubgraph().getNodes().size();
			} else {
				total += 1.0;
			}
		}
		precision = numberRelevants / total;
		return precision;
	}

	public double getRecall(Graph g, int totalNumberRelevants) {
		double recall = 0.0;
		Set<Node> nodes = g.getNodes();
		double numberRelevants = 0.0;
		for (Node n : nodes) {
			if (!n.isSubgraph()) {
				if (n.isRelevant()) {
					numberRelevants += 1.0;
				}
			} else {
				Set<Node> subnodes = n.getSubgraph().getNodes();
				for (Node nn : subnodes) {
					if (nn.isRelevant()) {
						numberRelevants += 1.0;
					}
				}
			}
		}
		double total = totalNumberRelevants;
		recall = numberRelevants / total;
		return recall;
	}

	public double getFmeasure(Graph g, int totalNumberRelevants, double fAlpha) {
		double fmeasure = 0.0;
		double recall = this.getRecall(g, totalNumberRelevants);
		double precision = this.getPrecision(g);
		fmeasure = precision * recall
				/ ((1 - fAlpha) * precision + fAlpha * recall);
		return fmeasure;
	}

	public double getPathRelevance(Graph g) {
		double relevance = 0.0;
		Set<Node> nodes = g.getNodes();
		double totalNodes = nodes.size();
		for (Node node : nodes) {
			relevance += node.getRelevance();
		}
		relevance = relevance / totalNodes;
		return relevance;
	}

	public void calculateNodesRelevancies(Graph graph,
			OntologyData ontologyData, double alpha, double beta) {
		this.calculateNodesRelevancies(graph, ontologyData, alpha, beta, 0);
	}

	public Graph getSummary() {
		Graph summary = null;
		this.calculateNodesRelevancies(this.originalGraph, ontologyData,
				rAlpha, rBeta, relevantNodesLimit);
		System.out
				.println("======================= RELEVANT NODES =======================");
		System.out.println();

		List<Node> relevantNodes = new ArrayList<Node>(this.originalGraph
				.getNodes());
		Collections.sort(relevantNodes);
		for (Node n : relevantNodes) {
			System.out.println("Node (" + n.getLabel() + ") relevance: "
					+ n.getRelevance());
		}
		System.out.println();
		System.out.println("Average Node Relevance: "
				+ this.getAverageNodeRelevance(originalGraph, ontologyData,
						fAlpha, rBeta));

		System.out.println();
		System.out
				.println("==================== END OF RELEVANT NODES ===================");

		Graph[] bestCasePath = this.joinNodesGraph(this.originalGraph,
				this.summarySizeReference);
		if (bestCasePath[0] != null) {
			summary = bestCasePath[0];
			summary = this.adjustSize(summary);
			summary.setFMeasure(this.getFmeasure(summary, relevantNodesLimit,
					fAlpha));
			summary.setRecall(this.getRecall(summary, relevantNodesLimit));
			summary.setPrecision(this.getPrecision(summary));
			System.out
					.println("\nAll relevant nodes are adjacents! No need to find summary candidates!\n");
		} else {
			this.originalGraph = bestCasePath[1];
			List<Node> rNodes = this.originalGraph.getRelevantNodes();
			if (rNodes.size() == 1) {
				this.originalGraph = this.unjoinGraph(this.originalGraph);
			}
			List<NodePair> pairNodes = new ArrayList<NodePair>();
			pairNodes = NodePair.get2x2Combinations(null, pairNodes, rNodes, 0,
					(rNodes.size() * (rNodes.size() - 1) / 2));
			Map<NodePair, Graph> pathMatrix = new HashMap<NodePair, Graph>();
			for (NodePair np : pairNodes) {
				Set<Graph> pathSet = this.getAllPaths(this.originalGraph, np
						.getSource(), np.getTarget());

				List<Graph> pathList = new ArrayList<Graph>(pathSet);

				pathList = this.constraintList(pathList,
						this.summarySizeReference, this.summarySizeDelta);

				this.sortPathList(pathList);
				if (pathList.size() > 0) {
					pathMatrix.put(np, pathList.get(0));
				}
				System.out
						.println("==================== COMPLETE PATH LIST ======================");
				System.out.println();
				for (Graph p : pathList) {
					p = this.unjoinGraph(p);
					p.setFMeasure(this.getFmeasure(p, relevantNodesLimit,
							fAlpha));
					p.setRecall(this.getRecall(p, relevantNodesLimit));
					p.setPrecision(this.getPrecision(p));
					System.out.println("Path: " + np);
					System.out.println("Nodes: " + p.getNodes());
					System.out.println("Edges: " + p.getEdges());
					System.out.println("Size: " + p.getSize(true));
					System.out.println("Recall: " + p.getRecall());
					System.out.println("Precision: " + p.getPrecision());
					System.out.println("F-Measure: " + p.getFMeasure());
					System.out.println("Relevance: " + p.getRelevance());
					System.out.println();
				}
				System.out
						.println("================= END OF COMPLETE PATH LIST ==================");
			}
			Set<NodePair> nps = pathMatrix.keySet();

			System.out
					.println("================== BEST SUMMARY CANDIDATES ===================");
			System.out.println();
			for (NodePair np : nps) {
				Graph path = pathMatrix.get(np);
				path = this.unjoinGraph(path);
				path.setFMeasure(this.getFmeasure(path, relevantNodesLimit,
						fAlpha));
				path.setRecall(this.getRecall(path, relevantNodesLimit));
				path.setPrecision(this.getPrecision(path));
				System.out.println("Path: " + np);
				System.out.println("Nodes: " + path.getNodes());
				System.out.println("Edges: " + path.getEdges());
				System.out.println("Size: " + path.getSize(true));
				System.out.println("Recall: " + path.getRecall());
				System.out.println("Precision: " + path.getPrecision());
				System.out.println("F-Measure: " + path.getFMeasure());
				System.out.println("Relevance: " + path.getRelevance());
				System.out.println();
				if (summary == null) {
					summary = path;
				} else {
					if (summary.compareTo(path) == 1) {
						summary = path;
					}
				}
				System.out.println();
			}
			System.out
					.println("=============== END OF BEST SUMMARY CANDIDATES ===============");
		}
		System.out.println();
		System.out
				.println("======================= SUMMARY CHOSEN =======================");
		System.out.println();
		if (summary != null) {
			System.out.println("Nodes: " + summary.getNodes());
			System.out.println("Edges: " + summary.getEdges());
			System.out.println("Size: " + summary.getSize(true));
			System.out.println("Recall: " + summary.getRecall());
			System.out.println("Precision: " + summary.getPrecision());
			System.out.println("F-Measure: " + summary.getFMeasure());
			System.out.println("Relevance: " + summary.getRelevance());
		} else {
			System.out
					.println("No summary found for the constraints informed.");
		}
		System.out.println();
		System.out
				.println("==================== END OF SUMMARY CHOSEN ===================");
		System.out.println();
		return summary;
	}

	private Graph adjustSize(Graph summary) {
		List<Node> nodes = new ArrayList<Node>(summary.getNodes());
		if (nodes.size() > this.summarySizeReference + this.summarySizeDelta) {
			Collections.sort(nodes);
			int expSize = this.summarySizeReference + this.summarySizeDelta;
			int dif = nodes.size() - expSize;
			for (int i = dif; i > 0; i--) {
				nodes.remove((expSize - 1) + i);
			}
		}
		return summary;
	}

	private Graph[] joinNodesGraph(Graph graph, int relevantNodesLimit) {
		Graph[] preProcess = new Graph[2];
		graph.hideAutoEdges();
		while (graph.containsAnyFullRelevantEdge()) {
			Set<Edge> edges = graph.getEdges();
			for (Edge edge : edges) {
				if (edge.isFullRelevant()) {
					graph = this.getJointNodesGraph(graph, edge);
					break;
				}
			}
		}
		preProcess[1] = graph;
		Set<Node> nodes = graph.getNodes();
		for (Node n : nodes) {
			if (n.isRelevant() && n.isSubgraph()
					&& (n.getSubgraph().getSize(true) >= relevantNodesLimit)) {
				preProcess[0] = this.unjoinGraph(n.getSubgraph());
				break;
			}
		}
		return preProcess;
	}

	private List<Graph> constraintList(List<Graph> pathList,
			int summarySizeReference, int summarySizeDelta) {
		List<Graph> list = new ArrayList<Graph>(pathList);
		for (Graph ppath : pathList) {
			Graph path = this.unjoinGraph(ppath);
			if ((path.getSize(true) > summarySizeReference + summarySizeDelta)
					|| (path.getSize(true) < summarySizeReference
							- summarySizeDelta)) {
				list.remove(ppath);
			}
		}
		return list;
	}

	private void sortPathList(List<Graph> pathList) {
		for (Graph p : pathList) {
			p.setFMeasure(this.getFmeasure(p, relevantNodesLimit, fAlpha));
			p.setRecall(this.getRecall(p, relevantNodesLimit));
			p.setPrecision(this.getPrecision(p));
		}
		Collections.sort(pathList);
	}
}
