package summarizer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class Graph implements Comparable<Graph> {

	private int id;
	private int directed;
	private String name;
	private Set<Edge> edges;
	private Set<Node> nodes;
	private Set<Edge> autoEdges;
	private double fMeasure;
	private double recall;
	private double precision;

	public Graph() {
		this.name = "Empty Graph";
		this.directed = 1;
		this.edges = new HashSet<Edge>();
		this.nodes = new HashSet<Node>();
		this.autoEdges = new HashSet<Edge>();
	}

	public boolean containsAnyFullRelevantEdge() {
		for (Edge e : this.edges) {
			if (e.isFullRelevant()) {
				return true;
			}
		}
		return false;
	}

	public boolean containsSubgraph() {
		for (Node n : this.nodes) {
			if (n.isSubgraph()) {
				return true;
			}
		}
		return false;
	}

	public List<Node> getRelevantNodes() {
		List<Node> rNodes = new ArrayList<Node>();
		for (Node node : this.nodes) {
			/*
			 * if (node.isSubgraph()) {
			 * rNodes.addAll(node.getSubgraph().getRelevantNodes()); } else {
			 */
			if (node.isRelevant()) {
				rNodes.add(node);
			}
			/* } */
		}
		Collections.sort(rNodes);
		return rNodes;
	}

	public void hideAutoEdges() {
		for (Edge e : this.edges) {
			if (e.isAutoEdge()) {
				this.autoEdges.add(e);
			}
		}
		this.edges.removeAll(this.autoEdges);
	}

	public void showAutoEdges() {
		for (Edge e : this.autoEdges) {
			this.edges.add(e);
		}
		this.autoEdges.clear();
	}

	public Graph getNormalizedGraph(boolean alreadyNormal) {
		if (alreadyNormal) {
			return this;
		}
		Graph graph = new Graph();
		graph.setName(this.getName());
		graph.getNodes().addAll(this.getNodes());
		graph.getEdges().addAll(this.getEdges());
		Set<Edge> edges = graph.getEdges();
		boolean completedNormalization = true;
		for (Edge e : edges) {
			if (e.isAutoEdge()) {
				graph.appendAutoEdgeToLabelNode(e.getSource().getId(), e
						.getLabel());
				completedNormalization = false;
				break;
			}
		}
		if (!completedNormalization) {
			graph.getNormalizedGraph(false);
		} else {
			graph.getNormalizedGraph(true);
		}
		return graph;
	}

	public void appendAutoEdgeToLabelNode(int id, String label) {
		for (Node n : this.nodes) {
			if (n.getId() == id) {
				n.setLabel(n.getLabel() + " (" + label + ")");
				break;
			}
		}
		for (Edge e : this.edges) {
			if ((e.getSource().getId() == id) && (e.getTarget().getId() == id)) {
				this.edges.remove(e);
				break;
			}
		}
	}

	public void removeEdgeByNode(Edge e) {
		this.edges.remove(e);
	}

	public void removeEdgeByNode(Node oldNode) {
		for (Edge e : this.edges) {
			if (e.getSource().equals(oldNode) || e.getTarget().equals(oldNode)) {
				this.edges.remove(e);
			}
		}
	}

	public void removeNodeByID(int id) {
		for (Node node : this.nodes) {
			if (node.getId() == id) {
				this.nodes.remove(node);
			}
		}
	}

	public void addEdge(Edge edge) {
		if (this.edges.size() > 0) {
			Edge nedge = null;
			for (Edge e : this.edges) {
				if (e.equals(edge)) {
					if (!e.getLabel().contains(edge.getLabel())) {
						e.addLabel(edge.getLabel());
						nedge = e;
						this.edges.remove(edge);
						break;
					}
				}
			}
			if (nedge != null) {
				this.edges.add(nedge);
			} else {
				this.edges.add(edge);
			}
		} else {
			this.edges.add(edge);
		}
	}

	public void eliminateAloneNodes() {
		Set<Node> nodes = new HashSet<Node>();
		for (Edge edge : this.edges) {
			nodes.add(edge.getSource());
			nodes.add(edge.getTarget());
		}
		this.nodes = nodes;
	}

	public Set<Edge> getNodeEdges(Node node) {
		Set<Edge> edges = new HashSet<Edge>();
		for (Edge edge : this.edges) {
			if (edge.getSource().equals(node) || edge.getTarget().equals(node)) {
				edges.add(edge);
			}
		}
		return edges;
	}

	public String getEdgeLabelByNodes(Node source, Node target) {
		String edgeLabel = "";
		for (Edge edge : edges) {
			if (edge.getSource().equals(source)
					&& edge.getTarget().equals(target)) {
				edgeLabel = edge.getLabel();
				break;
			}
		}
		return edgeLabel;
	}

	public Edge getEdgeByNodes(Node source, Node target) {
		Edge e = null;
		for (Edge edge : edges) {
			if (edge.getSource().equals(source)
					&& edge.getTarget().equals(target)) {
				e = edge;
				break;
			}
		}
		return e;
	}

	public Node getNodeByLabel(String name) {
		Node node = null;
		for (Iterator<Node> iter = nodes.iterator(); iter.hasNext();) {
			node = iter.next();
			if (node.getLabel().equals(name)) {
				break;
			}
		}
		return node;
	}

	public Node getNodeByID(int id) {
		Node node = null;
		for (Iterator<Node> iter = nodes.iterator(); iter.hasNext();) {
			node = iter.next();
			if (node.getId() == id) {
				break;
			}
		}
		return node;
	}

	public boolean addNode(Node node) {
		return this.nodes.add(node);
	}

	public int isDirected() {
		return directed;
	}

	public void setDirected(int directed) {
		this.directed = directed;
	}

	public Set<Edge> getEdges() {
		return edges;
	}

	public void setEdges(Set<Edge> edges) {
		this.edges = edges;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Set<Node> getNodes() {
		return nodes;
	}

	public void setNodes(Set<Node> nodes) {
		this.nodes = nodes;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getInverseDirectional(Node n1, Node n2) {
		String str = "";
		for (Edge e : edges) {
			if (e.getSource().equals(n2) && e.getTarget().equals(n1)) {
				str = n2.getLabel() + " -> " + n1.getLabel();
				break;
			}
		}
		return str;
	}

	@Override
	public String toString() {
		String graph = "";
		graph += this.getName() + " {\n";
		graph += "Vertices [ ";
		for (Node n : this.nodes) {
			graph += n.toString() + ", ";
		}
		graph = graph.substring(0, graph.length() - 2);
		graph += " ]\n";
		graph += "Edges [ ";
		for (Edge e : this.edges) {
			graph += e.toString() + ", ";
		}
		graph = graph.substring(0, graph.length() - 2);
		graph += " ]";
		graph += "\n}";
		return graph;
	}

	public boolean contains(Node node) {
		for (Node n : this.nodes) {
			if (n.equals(node)) {
				return true;
			}
		}
		return false;
	}

	public boolean contains(Edge edge) {
		for (Edge e : this.edges) {
			if (e.equals(edge)) {
				return true;
			}
		}
		return false;
	}

	public Set<Edge> getAutoEdges(Node node) {
		Set<Edge> autoedges = new HashSet<Edge>();
		for (Edge edge : this.edges) {
			if (edge.isAutoEdge() && edge.getSource().equals(node)) {
				autoedges.add(edge);
			}
		}
		return autoedges;
	}

	public void addEdges(Set<Edge> edges) {
		this.edges.addAll(edges);

	}

	public Graph getSubgraph() {
		Graph subgraph = null;
		for (Node node : this.nodes) {
			if (node.isSubgraph()) {
				Graph subgraph1 = node.getSubgraph();
				Graph subgraph2 = subgraph1.getSubgraph();
				if (subgraph2 != null) {
					subgraph = new Graph();
					subgraph.setName("Complete Subgraph");
					subgraph.addNodes(subgraph1.getNodes());
					subgraph.addNodes(subgraph2.getNodes());

				} else {
					subgraph = subgraph1;
				}
				break;
			}
		}
		return subgraph;
	}

	private void addNodes(Set<Node> nodes) {
		this.nodes.addAll(nodes);
	}

	public Graph getCompleteSubgraph() {
		Graph subgraph = null;
		for (Node node : this.nodes) {
			if (node.isSubgraph()) {
				subgraph = node.getSubgraph();
				break;
			}
		}
		return subgraph;
	}

	public Set<Edge> getEdgesByNode(Node node) {
		Set<Edge> edges = new HashSet<Edge>();
		for (Edge edge : this.edges) {
			if (edge.getSource().equals(node) || edge.getTarget().equals(node)) {
				edges.add(edge);
			}
		}
		return edges;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((edges == null) ? 0 : edges.hashCode());
		result = prime * result + ((nodes == null) ? 0 : nodes.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Graph other = (Graph) obj;
		if (edges == null) {
			if (other.edges != null) {
				return false;
			}
		} else if (!edges.equals(other.edges)) {
			return false;
		}
		if (nodes == null) {
			if (other.nodes != null) {
				return false;
			}
		} else if (!nodes.equals(other.nodes)) {
			return false;
		}
		return true;
	}

	public Graph getCopy() {
		Graph copy = new Graph();
		copy.setName(this.name);
		copy.setId(this.id);
		copy.setDirected(this.directed);
		for (Node node : this.nodes) {
			copy.addNode(node.getCopy());
		}
		for (Edge edge : this.edges) {
			copy.addEdge(edge.getCopy());
		}
		for (Edge autoEdge : this.autoEdges) {
			copy.getAutoEdges().add(autoEdge.getCopy());
		}
		return copy;
	}

	public int getAvaiableNodeID() {
		int avaiable = 1;

		for (Node n1 : this.nodes) {
			avaiable += n1.getId();
			if (n1.isSubgraph()) {
				Set<Node> nodes = n1.getSubgraph().getNodes();
				for (Node n2 : nodes) {
					avaiable += n2.getId();
				}
			}
		}
		return avaiable;
	}

	public Set<Edge> getAutoEdges() {
		return autoEdges;
	}

	public void setAutoEdges(Set<Edge> autoEdges) {
		this.autoEdges = autoEdges;
	}

	public int getDirected() {
		return directed;
	}

	private boolean pathFinded;

	public boolean isPathFinded() {
		return pathFinded;
	}

	public void setPathFinded(boolean pathFinded) {
		this.pathFinded = pathFinded;
	}

	public double getFMeasure() {
		return fMeasure;
	}

	public void setFMeasure(double measure) {
		fMeasure = measure;
	}

	public double getRecall() {
		return recall;
	}

	public void setRecall(double recall) {
		this.recall = recall;
	}

	public double getPrecision() {
		return precision;
	}

	public void setPrecision(double precision) {
		this.precision = precision;
	}

	public double getRelevance() {
		double relevance = 0.0;
		for (Node n : this.nodes) {
			relevance += n.getRelevance();
		}
		relevance = relevance / this.nodes.size();
		return relevance;
	}

	public int getSize(boolean consideringSubgraphs) {
		int size = 0;
		for (Node n : this.nodes) {
			if (consideringSubgraphs && n.isSubgraph()) {
				size += n.getSubgraph().getSize(consideringSubgraphs);
			} else {
				size += 1;
			}
		}
		return size;
	}

	public int compareTo(Graph o) {
		int comparision = -1;
		if ((o.getFMeasure() == this.fMeasure)
				&& (o.getRelevance() == this.getRelevance())
				&& (o.getSize(true) == this.getSize(true))) {
			comparision = 0;
		} else if (o.getFMeasure() > this.fMeasure) {
			comparision = 1;
		} else if ((o.getFMeasure() == this.fMeasure)
				&& (o.getRelevance() > this.getRelevance())) {
			comparision = 1;
		} else if ((o.getFMeasure() == this.fMeasure)
				&& (o.getRelevance() == this.getRelevance())
				&& (o.getSize(true) < this.getSize(true))) {
			comparision = 1;
		}
		return comparision;
	}

}
