package summarizer;

import java.util.Set;

public interface GraphSummaryCalculator {

	public Graph getSummary();

	public int getSummarySizeDelta();

	public void setSummarySizeDelta(int summarySizeDelta);

	public OntologyData getOntologyData();

	public void setOntologyData(OntologyData ontologyData);

	public Graph getOriginalGraph();

	public void setOriginalGraph(Graph originalGraph);

	public double getRAlpha();

	public void setRAlpha(double alpha);

	public double getRBeta();

	public void setRBeta(double beta);

	public double getFAlpha();

	public void setFAlpha(double alpha);

	public int getRelevantNodesLimit();

	public void setRelevantNodesLimit(int relevantNodesLimit);

	public int getSummarySizeReference();

	public void setSummarySizeReference(int summarySizeReference);

	public int getSymmarySizeDelta();

	public void setSymmarySizeDelta(int symmarySizeDelta);

	public int getNodeCentrality(Graph graph, Node node);

	public double getNodeNormalizedCentrality(Graph graph, Node node);

	public int getNodeFrequency(Graph graph, Node node,
			OntologyData ontologyData);

	public double getNodeNormalizedFrequency(Graph graph, Node node,
			OntologyData ontologyData);

	public double getNodeRelevance(Graph graph, Node node,
			OntologyData ontologyData, double alpha, double beta);

	public void calculateNodesRelevancies(Graph graph,
			OntologyData ontologyData, double alpha, double beta, int limit);

	public void calculateNodesRelevancies(Graph graph,
			OntologyData ontologyData, double alpha, double beta);

	public double getAverageNodeRelevance(Graph graph,
			OntologyData ontologyData, double alpha, double beta);

	public Set<Graph> getAllPaths(Graph original, Node source, Node target);

	/**
	 * Transforma os dois ns de uma aresta pertencente a um grafo G em um n e
	 * cria um grafo G' equivalente ao grafo G sem a aresta e com o n
	 * equivalendo a um subgrafo contendo a aresta.
	 * 
	 * @param originalGraph
	 *            Grafo original.
	 * @param edge
	 *            Aresta a ser transformada em n no novo grafo G'.
	 * 
	 * @return {@link Graph} Grafo G' equivalente  transformao da aresta em
	 *         n em G.
	 */
	public Graph getJointNodesGraph(Graph originalGraph, Edge edge);

	/**
	 * Faz o processo inverso do mtodo {@link getJointNodesGraph}, isto , G'
	 * -> G. E os ns so descondensados
	 * 
	 * @param graph
	 *            Grafo a ser descondesado.
	 * 
	 * @return {@link Graph} Grafo G' equivalente  transformao da aresta em
	 *         n em G.
	 */
	public Graph unjoinGraph(Graph jGraph);

	/**
	 * Calcula a cobertura de um caminho representado num grafo, onde:<br>
	 * <br>
	 * <i>recall = number of relevant concepts in a path / number of relevant
	 * concepts</i>
	 * 
	 * @param g
	 *            Grafo que representa o caminho do grafo original.
	 * @param totalNumberRelevants
	 *            Quantidade total de ns relevantes do grafo original.
	 * @return Valor da cobertura do caminho.
	 */
	public double getRecall(Graph g, int totalNumberRelevants);

	/**
	 * Calcula a preciso de um caminho representado num grafo, onde:<br>
	 * <br>
	 * <i>precision = number of relevant concepts in a path / path size.</i>
	 * 
	 * @param g
	 *            Grafo que representa o caminho.
	 * @return Valor da preciso do caminho.
	 */
	public double getPrecision(Graph g);

	/**
	 * Calcula a mdia harmnica alpha-paramtrica de um caminho representado
	 * num grafo, onde:<br>
	 * <br>
	 * <i>M(OS,RC,A) = Precision(OS,RC) x Recall(OS,RC) / [
	 * (1-A)xPrecision(OS,RC) + AxRecall(OS,RC) ]</i>
	 * 
	 * @param g
	 *            Grafo que representa o caminho do grafo original.
	 * @param totalNumberRelevants
	 *            Quantidade total de ns relevantes do grafo original.
	 * @param alpha
	 *            parmetro da mdia hamnica
	 * @return Valor da mdia harmnica alpha-paramtrica do caminho.
	 */
	public double getFmeasure(Graph g, int totalNumberRelevants, double alpha);

}
