package summarizer;

import java.net.URI;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.semanticweb.owl.apibinding.OWLManager;
import org.semanticweb.owl.model.OWLAxiom;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLDescription;
import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.OWLOntologyCreationException;
import org.semanticweb.owl.model.OWLOntologyManager;
import org.semanticweb.owl.util.SimpleURIMapper;

public class Ontology {

	private static final String RELATIONSHIP_IS_A = "is-a";
	private static final String CHAR_SEPARATOR = " ";
	public static final String RANGE_PROPERTY = "ObjectPropertyRange(";
	public static final String DOMAIN_PROPERTY = "ObjectPropertyDomain(";
	public static final String PROPERTY_MIN_CARDINALITY = "ObjectMinCardinality(";

	private OWLOntology ontology;

	public Ontology(OWLOntology owl) {
		this.ontology = owl;
	}

	public Ontology(String ontologyURIAddress, String physicalURIAddress)
			throws OWLOntologyCreationException {
		OWLOntologyManager manager = OWLManager.createOWLOntologyManager();

		URI ontologyURI = URI.create(ontologyURIAddress);

		URI physicalURI = URI.create(physicalURIAddress);

		SimpleURIMapper mapper = new SimpleURIMapper(ontologyURI, physicalURI);
		manager.addURIMapper(mapper);

		this.ontology = manager.createOntology(ontologyURI);
	}

	public Set<String> getConcepts() {
		Set<String> set = new HashSet<String>();
		Set<OWLClass> owlcSet = this.ontology.getReferencedClasses();
		Iterator<OWLClass> setIterator = owlcSet.iterator();
		while (setIterator.hasNext()) {
			OWLClass owlClass = setIterator.next();
			set.add(owlClass.toString());
		}
		return set;
	}

	public boolean containsTripleName(Set<Triple> triples, String tripleName) {
		boolean contains = false;
		if (triples != null) {
			for (Triple t : triples) {
				contains = t.getName().equals(tripleName);
				if (contains) {
					break;
				}
			}
		}
		return contains;
	}

	public void removeTripleByName(Set<Triple> triples, String tripleName) {
		if (triples != null) {
			for (Triple t : triples) {
				if (t.getName().equals(tripleName)) {
					triples.remove(t);
				}
			}
		}
	}

	public Set<Triple> getTriples() {
		Set<OWLAxiom> axs = this.ontology.getAxioms();
		String axiom = "";
		Set<Triple> triples = new HashSet<Triple>();
		for (OWLAxiom propertyAxiom : axs) {
			OWLAxiom ax = propertyAxiom;
			axiom = ax.toString();
			// System.out.println(axiom);
			if (axiom.contains(RANGE_PROPERTY)) {
				int index = axiom.indexOf(CHAR_SEPARATOR);
				String tripleName = axiom.substring(20, index);
				String targetConcept = axiom.substring(index + 1, axiom
						.length() - 1);
				Triple triple = new Triple(null, tripleName, targetConcept);
				if (this.containsTripleName(triples, tripleName)) {
					for (Triple tr : triples) {
						if (tr.getName().equals(tripleName)) {
							triple.setSourceConcept(tr.getSourceConcept());
							triples.remove(tr);
							triples.add(triple);
							break;
						}
					}
				} else {
					triples.add(triple);
				}
			} else if (axiom.contains(DOMAIN_PROPERTY)) {
				int index = axiom.indexOf(CHAR_SEPARATOR);
				String tripleName = axiom.substring(21, index);
				String sourceConcept = axiom.substring(index + 1, axiom
						.length() - 1);
				Triple triple = new Triple(sourceConcept, tripleName, null);
				if (this.containsTripleName(triples, tripleName)) {
					for (Triple tr : triples) {
						if (tr.getName().equals(tripleName)) {
							triple.setTargetConcept(tr.getTargetConcept());
							triples.remove(tr);
							triples.add(triple);
							break;
						}
					}
				} else {
					triples.add(triple);
				}
			}
		}
		Set<OWLClass> classes = this.ontology.getReferencedClasses();
		for (OWLClass owlClass : classes) {
			Set<OWLDescription> descriptions = owlClass
					.getSubClasses(this.ontology);
			for (OWLDescription owlDesc : descriptions) {
				Triple triple = new Triple(owlDesc.asOWLClass().toString(),
						RELATIONSHIP_IS_A, owlClass.toString());
				triples.add(triple);
			}
		}
		this.updateCardinalities(triples);
		/*
		 * for (Triple element : triples) { System.out.println(element);
		 * 
		 * }
		 */
		return triples;
	}

	public void updateCardinalities(Set<Triple> triples) {
		Set<OWLAxiom> axs = this.ontology.getAxioms();
		String axiom = "";
		for (OWLAxiom propertyAxiom : axs) {
			OWLAxiom ax = propertyAxiom;
			axiom = ax.toString();
			if (axiom.contains(PROPERTY_MIN_CARDINALITY)) {
				int index1 = axiom.indexOf(CHAR_SEPARATOR);
				String proprerty = axiom.substring(11, index1);
				String str3 = axiom.substring(index1 + 1, axiom.length());
				int index2 = str3.indexOf(CHAR_SEPARATOR);
				String str4 = str3.substring(index2 + 1, str3.length());
				int index3 = str4.indexOf(CHAR_SEPARATOR);
				String relation = str4.substring(0, index3);
				Set<Triple> newTriples = new HashSet<Triple>();
				newTriples = this.getTargetCardinalities(triples, newTriples,
						proprerty, relation);
				triples.addAll(newTriples);
			}
		}
	}

	public Set<Triple> getTargetCardinalities(Set<Triple> triples,
			Set<Triple> newTriples, String property, String relation) {
		Set<Triple> triplesConverted = new HashSet<Triple>();
		triplesConverted.addAll(triples);
		String concept = null;
		for (Triple triple : triplesConverted) {
			if (triple.getSourceConcept().equals(property)
					&& triple.getName().equals(RELATIONSHIP_IS_A)) {
				concept = triple.getTargetConcept();
				triplesConverted.remove(triple);
				break;
			}
		}
		if (concept != null) {
			for (Triple triple : triplesConverted) {
				if (triple.getSourceConcept().equals(concept)
						&& triple.getName().equals(relation)) {
					newTriples.add(new Triple(property, relation, triple
							.getTargetConcept()));
					break;
				}
			}
			newTriples = getTargetCardinalities(triplesConverted, newTriples,
					property, relation);
		}
		return newTriples;
	}

	public OWLOntology getOntology() {
		return ontology;
	}

	public void setOntology(OWLOntology owl) {
		this.ontology = owl;
	}

	public String getName() {
		String name = "";
		if (this.ontology != null) {
			name = ontology.toString();
		}
		return name;
	}

}
