package summarizer;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URL;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import org.semanticweb.owl.apibinding.OWLManager;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.OWLOntologyChangeException;
import org.semanticweb.owl.model.OWLOntologyCreationException;
import org.semanticweb.owl.model.OWLOntologyManager;
import org.semanticweb.owl.model.OWLOntologyStorageException;
import org.semanticweb.owl.model.UnknownOWLOntologyException;
import org.semanticweb.owl.util.OWLEntityRemover;

public class Summarizer {

	private static final String FILE_URI_TAG = "file:///";
	private GraphSummaryCalculator gcs;
	private String originalOntologyPath;
	private String summarizedOntologyPath;
	private String frenquencyFilePath;
	private double rAlpha;
	private double rBeta;
	private double fAlpha;
	private int relevantNodesLimit;
	private int summarySizeReference;
	private int summarySizeDelta;

	public Summarizer() {
		this.gcs = new FMeasureGraphCalculator();
	}

	private void summarize() {
		try {
			double tinicio = new Date().getTime();
			this.duplicateOriginalOntologyFile();
			OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
			URI physicalURI = URI.create(FILE_URI_TAG
					+ this.summarizedOntologyPath.replace("\\", "/"));
			OWLOntology owl = manager.loadOntologyFromPhysicalURI(physicalURI);
			Ontology ontology = new Ontology(owl);
			O2GParser o2g = new O2GParser(ontology);
			o2g.parse();
			Graph originalGraph = o2g.getGraph();
			this.gcs.setOriginalGraph(originalGraph);
			if (this.frenquencyFilePath != null) {
				XMLOntologyDataParser xodp = new XMLOntologyDataParser(
						this.frenquencyFilePath);
				xodp.parse();
				this.gcs.setOntologyData(xodp.getOntologyData());
			}
			this.startLogging();
			Graph summarizedGraph = this.gcs.getSummary();
			Set<String> classesToRemove = this.getNodesDifference(
					originalGraph, summarizedGraph);
			this.updateResult(classesToRemove, manager, owl);
			double ttermino = new Date().getTime();
			System.out.println();
			System.out.println("Total time: " + (ttermino - tinicio) / 1000
					+ " seconds.");
		} catch (OWLOntologyCreationException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void startLogging() throws IOException {
		String logFileName = this.originalOntologyPath.replace(".owl", ".log");
		File logFile = new File(logFileName);
		logFile.delete();
		logFile = new File(logFileName);
		logFile.createNewFile();

		LogStdStreams.initializeErrorLogging(logFileName,
				"Log File for ExampleLoggingApplication " + new Date(), true,
				true);
		System.out.println();
	}

	private void updateResult(Set<String> classesToRemove,
			OWLOntologyManager man, OWLOntology owl) {
		try {

			OWLEntityRemover remover = new OWLEntityRemover(man, Collections
					.singleton(owl));
			System.out.println("Original number of classes: "
					+ owl.getReferencedClasses().size());

			for (OWLClass owlClass : owl.getReferencedClasses()) {
				if (classesToRemove.contains(owlClass.toString())) {
					owlClass.accept(remover);
				}
			}

			man.applyChanges(remover.getChanges());
			System.out.println("Summarized number of classes: "
					+ owl.getReferencedClasses().size());

			remover.reset();
			man.saveOntology(owl);
		} catch (OWLOntologyChangeException e) {
			System.out.println("Problem applying changes to ontology: "
					+ e.getMessage());
		} catch (UnknownOWLOntologyException e) {
			e.printStackTrace();
		} catch (OWLOntologyStorageException e) {
			e.printStackTrace();
		}
	}

	private void duplicateOriginalOntologyFile() throws IOException {
		URL url = ClassLoader.getSystemResource(this.originalOntologyPath);
		if (url == null) {
			System.out.println("Arquivo " + this.originalOntologyPath
					+ " de ontologia no encontrado!");
			System.exit(1);
		}
		File src = new File(url.getPath());
		File dst = new File(this.summarizedOntologyPath);
		dst.createNewFile();

		InputStream in = new FileInputStream(src);
		OutputStream out = new FileOutputStream(dst);

		byte[] buf = new byte[1024];
		int len;
		while ((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}
		in.close();
		out.close();
		this.setSummarizedOntologyPath(dst.getAbsolutePath());
	}

	private Set<String> getNodesDifference(Graph originalGraph,
			Graph summarizedGraph) {
		Set<Node> nodesDifference = new HashSet<Node>();
		Set<String> classesToRemove = new HashSet<String>();
		if (summarizedGraph != null) {
			Set<Node> originalNodes = originalGraph.getNodes();
			nodesDifference.addAll(originalNodes);
			Set<Node> summarizedNodes = summarizedGraph.getNodes();
			nodesDifference.removeAll(summarizedNodes);
			for (Node node : nodesDifference) {
				classesToRemove.add(node.getLabel());
			}
		}
		return classesToRemove;
	}

	public GraphSummaryCalculator getGcs() {
		return gcs;
	}

	public void setGcs(GraphSummaryCalculator gcs) {
		this.gcs = gcs;
	}

	public String getOriginalOntology() {
		return originalOntologyPath;
	}

	public void setOriginalOntologyPath(String originalOntology) {
		this.originalOntologyPath = originalOntology;
	}

	public String getSummarizedOntology() {
		return summarizedOntologyPath;
	}

	public void setSummarizedOntologyPath(String summarizedOntology) {
		this.summarizedOntologyPath = summarizedOntology;
	}

	public String getFrenquencyFile() {
		return frenquencyFilePath;
	}

	public void setFrenquencyFilePath(String frenquencyFile) {
		this.frenquencyFilePath = frenquencyFile;
	}

	public double getRAlpha() {
		return rAlpha;
	}

	public void setRAlpha(double alpha) {
		rAlpha = alpha;
		this.gcs.setRAlpha(alpha);
	}

	public double getRBeta() {
		return rBeta;
	}

	public void setRBeta(double beta) {
		rBeta = beta;
		this.gcs.setRBeta(beta);
	}

	public double getFAlpha() {
		return fAlpha;
	}

	public void setFAlpha(double alpha) {
		this.gcs.setFAlpha(alpha);
		fAlpha = alpha;
	}

	public int getRelevantNodesLimit() {
		return relevantNodesLimit;
	}

	public void setRelevantNodesLimit(int relevantNodesLimit) {
		this.gcs.setRelevantNodesLimit(relevantNodesLimit);
		this.relevantNodesLimit = relevantNodesLimit;
	}

	public int getSummarySizeReference() {
		return summarySizeReference;
	}

	public void setSummarySizeReference(int summarySizeReference) {
		this.gcs.setSummarySizeReference(summarySizeReference);
		this.summarySizeReference = summarySizeReference;
	}

	public int getSummarySizeDelta() {
		return summarySizeDelta;
	}

	public void setSummarySizeDelta(int summarySizeDelta) {
		this.gcs.setSummarySizeDelta(summarySizeDelta);
		this.summarySizeDelta = summarySizeDelta;
	}

	public static void main(String[] args) {
		Summarizer s = new Summarizer();
		if (args.length == 9) {
			s.setOriginalOntologyPath(ClassLoader.getSystemResource(args[0])
					.getFile());
			s.setSummarizedOntologyPath(ClassLoader.getSystemResource(args[1])
					.getFile());
			s.setFrenquencyFilePath(ClassLoader.getSystemResource(args[2])
					.getFile());
			s.setRAlpha(Double.valueOf(args[3]));
			s.setRBeta(Double.valueOf(args[4]));
			s.setFAlpha(Double.valueOf(args[5]));
			s.setRelevantNodesLimit(Integer.valueOf(args[6]));
			s.setSummarySizeReference(Integer.valueOf(args[7]));
			s.setSummarySizeDelta(Integer.valueOf(args[8]));
			s.summarize();
		} else if (args.length == 8) {
			s.setOriginalOntologyPath(ClassLoader.getSystemResource(args[0])
					.getFile());
			s.setSummarizedOntologyPath(ClassLoader.getSystemResource(args[1])
					.getFile());
			s.setRAlpha(Double.valueOf(args[2]));
			s.setRBeta(Double.valueOf(args[3]));
			s.setFAlpha(Double.valueOf(args[4]));
			s.setRelevantNodesLimit(Integer.valueOf(args[5]));
			s.setSummarySizeReference(Integer.valueOf(args[6]));
			s.setSummarySizeDelta(Integer.valueOf(args[7]));
			s.summarize();
		} else if (args.length == 3) {
			s.setOriginalOntologyPath(args[0]);
			s.setSummarizedOntologyPath(args[1]);
			s.setFrenquencyFilePath(args[2]);
			Properties properties = new Properties();
			try {
				properties.load(ClassLoader
						.getSystemResourceAsStream("summary.properties"));
				s.setRAlpha(Double.valueOf(properties
						.getProperty("centrality_weight")));
				s.setRBeta(Double.valueOf(properties
						.getProperty("frequency_weight")));
				s.setFAlpha(Double.valueOf(properties
						.getProperty("alpha_measure")));
				s.setRelevantNodesLimit(Integer.valueOf(properties
						.getProperty("relevant_concepts")));
				s.setSummarySizeReference(Integer.valueOf(properties
						.getProperty("summary_size_reference")));
				s.setSummarySizeDelta(Integer.valueOf(properties
						.getProperty("summary_size_delta")));

			} catch (IOException e) {
				System.out.println(e);
			}
			s.summarize();
		} else if (args.length == 2) {
			s.setOriginalOntologyPath(args[0]);
			s.setSummarizedOntologyPath(args[1]);
			Properties properties = new Properties();
			try {
				properties.load(ClassLoader
						.getSystemResourceAsStream("summary.properties"));
				s.setRAlpha(Double.valueOf(properties
						.getProperty("centrality_weight")));
				s.setRBeta(Double.valueOf(properties
						.getProperty("frequency_weight")));
				s.setFAlpha(Double.valueOf(properties
						.getProperty("alpha_measure")));
				s.setRelevantNodesLimit(Integer.valueOf(properties
						.getProperty("relevant_concepts")));
				s.setSummarySizeReference(Integer.valueOf(properties
						.getProperty("summary_size_reference")));
				s.setSummarySizeDelta(Integer.valueOf(properties
						.getProperty("summary_size_delta")));

			} catch (IOException e) {
				System.out.println(e);
			}
			s.summarize();
		} else {
			System.out.println("OWL Ontology Summarizer");
			System.out.println();
			System.out.println("Syntax:");
			System.out.println();
			System.out.println("java -jar OWLSummarizer.jar <owl_filename>"
					+ " <owl_summarized_filename> <frenquency_filename>"
					+ " <centrality_weight> <frequency_weight> <alpha_measure>"
					+ " <relevant_concepts> <summary_size_reference>"
					+ " <summary_size_delta>");
			System.out.println();
			System.out.println("java -jar OWLSummarizer.jar <owl_filename>"
					+ " <owl_summarized_filename> <frenquency_filename>");
			System.out.println();
			System.out
					.println("WARNING: When passing only 3 parameters, you should\n"
							+ "define the others parameters in a property file\n"
							+ "called summary.properties visible in the classpath.");
			System.out.println();
			System.out.println("Sample content for a summary.properties:");
			System.out.println();
			System.out.println("centrality_weight=0.5");
			System.out.println("frequency_weight=0.5");
			System.out.println("alpha_measure=0.5");
			System.out.println("relevant_concepts=8");
			System.out.println("summary_size_reference=10");
			System.out.println("summary_size_delta=3");

		}
	}

}
